(ns clojureDemo.Xuggle)

(import '(ij ImagePlus IJ))
(import '(java.awt Rectangle Robot Toolkit) )
(import '(java.awt.image BufferedImage) )
(import '(java.awt Graphics2D Panel))
(import '(java.io File) )
(import '(javax.imageio ImageIO) )
(import '(com.xuggle.mediatool ToolFactory))
(import '(com.xuggle.mediatool IMediaDebugListener IMediaDebugListener$Event))
(import '(com.xuggle.mediatool MediaToolAdapter MediaListenerAdapter))
(import '(com.xuggle.xuggler IContainer IContainer$Type IPacket))
(import '(javax.swing JFrame))
(import '(com.xuggle.mediatool IMediaWriter))
(import '(org.apache.commons.io FileUtils))
(import '(javax.imageio.stream FileImageOutputStream))
(import '(javax.imageio ImageWriteParam IIOImage))
(import '(com.xuggle.xuggler IRational))
(import '(java.util.concurrent TimeUnit))
(import '(com.xuggle.xuggler ICodec))

(use  'clojureDemo.Defines)
;(use '[clojureDemo.OpenCv :only (video-data)])



(import '(java.io File))
(import '(org.apache.commons.io FileUtils))
(import '(javax.imageio ImageIO) )

(import '(ij.plugin PlugIn))
(import '(ij ImagePlus IJ))


(use 'clojure.contrib.repl-utils)
(use ['clojureDemo.Defines 
      :only '(
	     lian look getto human0 blow base app0 app1 app2 app3 app4 app5 
		  bounce0 bounce1 bounce2 bounce3 bounce4 bounce5 bounce6
		  collide0 collide1 collide2 collide3 collide4  
		  give0 give1 give2 give3 give4 target default)])


;(def hamster (ImagePlus. "lklk" (first (video-seq look))))





(defn- makePanel [image] (proxy [Panel] [] (paint [g]  (.drawImage g image 0 0 nil))))


(defmulti  display "Creates a JFrame and displays a buffered image"  class)

(defmethod display 
  BufferedImage  [image] 
  (let [panel (makePanel image)
	frame (JFrame. "Oh Yeah!")]
    (.add frame panel) 
    (.pack frame) 
    (.setVisible frame true ) 
    (.setSize frame(.getWidth image) (.getHeight image))))
 
(defmethod display
  ImagePlus [image]
  (display (.getBufferedImage image)))




(defn flash
     [image]
     
     (.start (Thread. (fn [] 
			(do 
			  (.show image)
			  (.updateAndRepaintWindow image)
			  (Thread/sleep 4000)
			  (.hide image))))))


(defn readerRecurse
  "calls .readPacket until there's nothing left to do"
  [reader]
  (if (not (nil? (.readPacket reader))) ; here .readPacket actually does the processing as a side-effect.
    nil                                   ; it returns null when it has MORE to process, and signals an error when done... 
    (recur reader)))


(def *cache-directory* (File. "/home/r/Desktop/vision-cache"))
(def *ext* "jpg")




(defn writeJpg
  "WTF is this shit?!"
  [image target quality]
  (let [jpgWriter (.next (ImageIO/getImageWritersByFormatName *ext*))]
    (doto (.getDefaultWriteParam jpgWriter) 
      (.setCompressionMode ImageWriteParam/MODE_EXPLICIT)
      (.setCompressionQuality quality))
    (doto jpgWriter
      (.setOutput (FileImageOutputStream. target))
      (.write (IIOImage. image nil nil))
      (.dispose))))
    


(defn cache-path
  [video]
  (File. *cache-directory* (.getName video)))

(defn video-frame-path
     [video n]
     (File. (cache-path video)  (str n "."  *ext*)))


(defn already-cached
  "this is the simplest and most retarded way to do it"
  [video]
  (.exists (cache-path video)))


 
 


(defn make-incrementer [start increment] (let [a (ref (- start increment))] (fn [] (dosync (ref-set a (+ @a increment))))))

(defn make-frame-writer
  [video]
  (let [incrementer (make-incrementer 0 1)]
  (proxy [MediaListenerAdapter] [] 
    
    (onVideoPicture
     [event]
     ;(println (.getImage event))
     ;(println (File. (cache-path video) (str (incrementer) "." *ext* ) ))
     
     (let [target  (File. (cache-path video) (str (incrementer) "." *ext* ))]
       (if (= *ext* "jpg")
	 (writeJpg (.getImage event)  target  1)
	 (ImageIO/write (.getImage event) *ext* target )))))))
    


(defn cache 
  "caching of frames without opencv"
  [video]
  
  (if (already-cached video)
    nil 
    (time
    (let [reader (ToolFactory/makeReader (str video))]
      (println "slow cache!")
      (FileUtils/forceMkdir (cache-path video))
      (doto reader
	(.setBufferedImageTypeToGenerate BufferedImage/TYPE_3BYTE_BGR)
	(.addListener  (make-frame-writer video))
	(.addListener (ToolFactory/makeDebugListener (into-array [IMediaDebugListener$Event/META_DATA]))))
      (readerRecurse reader)))))


(defn video-data 
  "get video data without opencv"
  [video] (cache video)

  
  {:length (- (count (file-seq (cache-path video))) 1)
   :width  (.getWidth (ImagePlus. (str (video-frame-path video 0))))
   :height (.getHeight (ImagePlus. (str (video-frame-path video 0))))
   :fps 30}) ; yeah --- I'll figure this out later.
 (def video-data (memoize video-data))



(defn convert
  "takes video and converts it to a new type of video"
  [videoInput videoOutput]
  (let [reader (ToolFactory/makeReader (str videoInput))]
    (doto reader
      (.addListener (ToolFactory/makeWriter (str videoOutput) reader))
      (.addListener (ToolFactory/makeDebugListener (into-array [IMediaDebugListener$Event/META_DATA]))))
    (readerRecurse reader)))


(import '(com.xuggle.xuggler.video ConverterFactory))


(defn trans-save
  "this is a transitive way to save a stream to an avi file.
   It uses meta data to determine what fps to use to encode."
  [destination video-seq]
  (let [data (meta video-seq)
	writer (ToolFactory/makeWriter (str destination))
	incrementer (make-incrementer 0 (/ 1 30))]
    (.addVideoStream writer 0 0 (ICodec/findEncodingCodecByName "mpeg4") 
		     (IRational/make (double (:fps data))) 
		     (int (:width data)) (int (:height data)))
    (dorun (map  #(.encodeVideo writer 0 
			 (ConverterFactory/convertToType (.getBufferedImage %) BufferedImage/TYPE_3BYTE_BGR) 
			(long (* 1000000000 (incrementer)))  TimeUnit/NANOSECONDS)  video-seq))
	(.close writer))
  video-seq)


(defn video-seq 
  "let's use ImagePlus stuff!"
  ([video] (cache video) 
    (with-meta (map #(ImagePlus. (str (video-frame-path video %))) (range (:length (video-data video))) ) (video-data video))))






(comment 
  (do (use :reload-all 'clojureDemo.Xuggle) (in-ns 'clojureDemo.Xuggle))
)
 
